"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeviceHandler = void 0;
const isomorphic_ws_1 = __importDefault(require("isomorphic-ws"));
const tiny_typed_emitter_1 = require("tiny-typed-emitter");
const deviceMessages_1 = require("./deviceMessages");
const logging_1 = require("./logging");
const webrtc_connection_1 = require("./peer/webrtc-connection");
const websocket_connection_1 = require("./peer/websocket-connection");
class DeviceHandler extends tiny_typed_emitter_1.TypedEmitter {
    constructor() {
        super(...arguments);
        this.connections = new Map();
        this.services = new Map();
        this.supportedConnectionTypes = ['webrtc'];
    }
    async connect(connectOptions) {
        this.ws = new isomorphic_ws_1.default(connectOptions.endpoint);
        this.ws.onopen = () => {
            this.ws.send(JSON.stringify({
                messageType: 'authenticate',
                deviceUrl: connectOptions.id,
                token: connectOptions.token,
            }));
            logging_1.crosslabTransport._set_upstream(info => this.ws.send(JSON.stringify({ messageType: 'logging', content: info })));
        };
        const p = new Promise((resolve, reject) => {
            this.ws.onmessage = authenticationEvent => {
                const authenticationMessage = JSON.parse(authenticationEvent.data);
                console.log(JSON.stringify(authenticationMessage, null, 4));
                if (authenticationMessage.messageType === 'authenticate') {
                    if (authenticationMessage.authenticated) {
                        resolve();
                    }
                    else
                        reject('Authentication failed');
                }
                else {
                    reject(`Expected message with messageType 'authenticate', received ${authenticationMessage.messageType}`);
                }
            };
        });
        this.ws.onclose = event => {
            logging_1.logger.log('info', 'ws closed', { reason: event.reason, code: event.code });
        };
        this.ws.onerror = event => {
            logging_1.logger.log('error', event.message, { type: event.type, error: event.error });
        };
        await p;
        this.ws.onmessage = event => {
            const message = JSON.parse(event.data);
            console.log(JSON.stringify(message, null, 4));
            if ((0, deviceMessages_1.isCommandMessage)(message)) {
                if ((0, deviceMessages_1.isCreatePeerConnectionMessage)(message)) {
                    this.handleCreatePeerConnectionMessage(message);
                }
                else if ((0, deviceMessages_1.isClosePeerConnectionMessage)(message)) {
                    this.handleClosePeerConnectionMessage(message);
                }
            }
            if ((0, deviceMessages_1.isSignalingMessage)(message)) {
                this.handleSignalingMessage(message);
            }
            if ((0, deviceMessages_1.isConfigurationMessage)(message)) {
                this.handleConfigurationMessage(message);
            }
            if ((0, deviceMessages_1.isExperimentStatusChangedMessage)(message)) {
                this.handleExperimentStatusChangedMessage(message);
            }
        };
    }
    addService(service) {
        this.services.set(service.serviceId, service);
    }
    handleCreatePeerConnectionMessage(message) {
        if (this.connections.has(message.connectionUrl)) {
            throw Error('Can not create a connection. Connection Id is already present');
        }
        console.log(JSON.stringify(message, null, 4));
        const connection = message.connectionType === 'webrtc'
            ? new webrtc_connection_1.WebRTCPeerConnection(message.connectionOptions)
            : new websocket_connection_1.WebSocketPeerConnection({ url: message.connectionOptions.webSocketUrl });
        connection.tiebreaker = message.tiebreaker;
        this.connections.set(message.connectionUrl, connection);
        for (const serviceConfig of message.services) {
            const service = this.services.get(serviceConfig.serviceId);
            if (service === undefined) {
                throw Error('No Service for the service config was found');
            }
            service.setupConnection(connection, serviceConfig);
        }
        connection.on('signalingMessage', msg => {
            logging_1.logger.log('info', 'sending:', msg);
            this.ws.send(JSON.stringify({
                ...msg,
                messageType: 'signaling',
                connectionUrl: message.connectionUrl,
            }));
        });
        connection.on('connectionChanged', () => {
            const connectionStateChangedMessage = {
                messageType: 'connection-state-changed',
                status: connection.state,
                connectionUrl: message.connectionUrl,
            };
            this.ws.send(JSON.stringify(connectionStateChangedMessage));
            this.emit('connectionsChanged');
        });
        connection.connect();
        this.emit('connectionsChanged');
    }
    handleSignalingMessage(message) {
        const connection = this.connections.get(message.connectionUrl);
        if (connection === undefined) {
            throw Error('No Connection for the signaling message was found');
        }
        connection.handleSignalingMessage(message);
    }
    handleClosePeerConnectionMessage(message) {
        const connection = this.connections.get(message.connectionUrl);
        if (!connection) {
            throw Error('Cannot close a connection. Connection Id is not present');
        }
        logging_1.logger.log('info', 'closing connection', message);
        connection.teardown();
        this.connections.delete(message.connectionUrl);
    }
    handleConfigurationMessage(message) {
        this.emit('configuration', message.configuration);
    }
    handleExperimentStatusChangedMessage(message) {
        this.emit('experimentStatusChanged', {
            status: message.status,
            message: message.message,
        });
    }
    getServiceMeta() {
        return Array.from(this.services.values()).map(service => {
            const meta = service.getMeta();
            return {
                ...meta,
                supportedConnectionTypes: meta.supportedConnectionTypes.filter(supportedConnectionType => this.supportedConnectionTypes.includes(supportedConnectionType)),
            };
        });
    }
}
exports.DeviceHandler = DeviceHandler;
//# sourceMappingURL=deviceHandler.js.map